
/* RCS Information:
 *
 * $Id$
 * $Revision$
 * $Date$
 * $Author$
 * $Locker$
 * 
 */

/******************************************************************************
 *
 *  Copyright (C) 2002, Bill Pierce
 *  This file is part of the hba_scsi_inquiry library.
 *
 *  The hba_scsi_inquiry library is free software; you can 
 *  redistribute it and/or modify it under the terms of the 
 *  GNU Library General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The hba_scsi_inquiry library is distributed in the hope that 
 *  it will be useful, but WITHOUT ANY WARRANTY; without even 
 *  the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 *  PARTICULAR PURPOSE.  See the GNU Library General Public 
 *  License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with the timeout action libary; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Bill Pierce can be contacted at:
 *  7862 NE 148th St.
 *  Kenmore, WA 98028
 *
 * Name -- hba_scsi_inquiry.h 
 *
 * Purpose -- header file to describe the library routines of
 *            hba_scsi_inquiry.c  These routines assist in the 
 *            development of applications that use scsi_inquiry
 *            functionality provided by the SNIA HBA API. 
 *
 * Remarks --  
 *
 * Known bugs --  
 *
 * Supported Platforms -- Windows NT, 2000, UNIX flavors. 
 *
 * Applicable Invariants -- 
 * 
 * Language Version --  Standard C  
 *
 * Author --  Bill Pierce 
 *
 * Origination Date -- 8-21-01
 * 
 *****************************************************************************/

#include "hbaapi.h"  /* The SNIA HBA API */

/* Default timeout value for scsi inquiry polls. */
#define DEFAULT_INQUIRY_SYNCHRONOUS_WAIT_LIMIT_MS 1000

/* Scsi device type definitions. */
#define SCSI_DEVICE_TYPE_DIRECT_ACCESS                0x00 
#define SCSI_DEVICE_TYPE_SEQUENTIAL_ACCESS            0x01 
#define SCSI_DEVICE_TYPE_PRINTER                      0x02 
#define SCSI_DEVICE_TYPE_PROCESSOR                    0x03 
#define SCSI_DEVICE_TYPE_WRITE_ONCE                   0x04 
#define SCSI_DEVICE_TYPE_CD_ROM                       0x05 
#define SCSI_DEVICE_TYPE_SCANNER                      0x06 
#define SCSI_DEVICE_TYPE_OPTICAL_MEMORY               0x07 
#define SCSI_DEVICE_TYPE_MEDIUM_CHANGER               0x08 
#define SCSI_DEVICE_TYPE_COMMNUNICATIONS              0x09 
#define SCSI_DEVICE_TYPE_STORAGE_ARRAY_CONTROLLER     0x0C 
#define SCSI_DEVICE_TYPE_ENCLOSURE_SERVICES           0x0D 
#define SCSI_DEVICE_TYPE_UNKNOWN                      0x1F 

/* Scsi inquiry reply status indicators. */
#define SCSI_INQUIRY_REPLY_SUCCESS   0   /* All's well */
#define SCSI_INQUIRY_REPLY_TIMED_OUT 1   /* Timeout */
#define SCSI_INQUIRY_REPLY_EMPTY     2   /* Got empty response */
#define SCSI_INQUIRY_REPLY_ERROR     3   /* Error while trying */

/* Structure to hold the byte response to a scsi inquiry request. */
typedef struct ScsiInquiryResponse /* Standard Inquiry response format */
{
    HBA_INT8 HeaderByteZero;      /* Byte zero containes the device type.  */
    HBA_INT8 HeaderRemainder[7];  /* Not useful in this context. */
    HBA_INT8 VendorId[8];         /* Vendor ID */
    HBA_INT8 ProductId[16];       /* Product ID */
    HBA_INT8 Revision[4];         /* Product revision */
    HBA_INT8 VendorSpec[20];      /* Vendor specific */
} SCSI_INQUIRY_RESPONSE, *P_SCSI_INQUIRY_RESPONSE;
  
/* A macro used in extracting the scsi device type from HeaderByteZero. */
#define LOWER_FOUR_BITS_OF(x) ((x) & (0x0F))

/* Perform a scsi inquiry.  This method is synchronous, but guaranteed
 * to return within timeout_in_ms milliseconds.
 * Return one of the SCSI_INQUIRY_REPLY status values to indicate 
 * success or failure.  If successful, the inquiry response structure
 * will be written to p_inquiry_response.
 */
int do_synchronous_time_limited_inquiry(
                HBA_HANDLE              adapter_handle,
                HBA_WWN                 target_port_wwn,
                HBA_UINT64              fcp_lun,
                unsigned int            timeout_in_ms,                     
                P_SCSI_INQUIRY_RESPONSE p_inquiry_response
		                       );

/* Perform a scsi inquiry.  This method is synchronous 
 * and will block until the SendScsiInqury() method 
 * in the HBA API returns.  The calling thread to 
 * this method is at the mercy of the library.
 * Return one of the SCSI_INQUIRY_REPLY status values to indicate 
 * success or failure. If successful, the inquiry response 
 * structure will be written to p_inquiry_response. 
 */
int do_synchronous_inquiry(
                HBA_HANDLE              adapter_handle,
                HBA_WWN                 target_port_wwn,
                HBA_UINT64              fcp_lun,
                P_SCSI_INQUIRY_RESPONSE p_inquiry_response
		          );

/* Write that a SCSI Inqiry ping is going out to the target 
 * specified by port_wwn and the LUN specified by
 * fcp_lun.
 */
void print_inquiry_going_out(HBA_WWN port_wwn, HBA_UINT64 fcp_lun);

/* Write that a reply was received from the scsi device at 
 * port_wwn. p_scsi_resonse should contain the vendor and 
 * product name of the target, and response time contains
 * the time in seconds that it took for the inquiry call
 * to return.
 */
void print_ping_reply_received(
                          HBA_WWN                 port_wwn, 
		          P_SCSI_INQUIRY_RESPONSE p_scsi_response,
                          double                  response_time
                          );

/* Write that the scsi inquiry request timed out. */
void print_ping_request_timed_out();

/* Write that the scsi inquiry response was empty. */
void print_ping_request_reply_empty();

/* Write that the scsi inquiry response was unsuccessful. */
void print_ping_request_unsuccessful();

/*****************************************************************************
 *
 *   $Log: $
 *
 *****************************************************************************/
